import java.io.*;
import java.nio.file.*;
import java.nio.channels.FileChannel;

public class NIO2CopyAFile {

    public static void main(String... args) throws IOException {
        String from = "c:/windows/notepad.exe";
        String toIO = "io.exe";
        String toNIO = "nio.exe";
        String toNIO2 = "nio2.exe";

        copyIO(from, toIO);
        copyNIO(from, toNIO);
        copyNIO2(from, toNIO2);
    }

    public static void copyIO(String fromFileName, String toFileName)
            throws IOException {
        FileInputStream from = null;
        FileOutputStream to = null;
        try {
            from = new FileInputStream(new File(fromFileName));
            to = new FileOutputStream(new File(toFileName));
            byte[] buffer = new byte[4096];
            int bytesRead;

            while ((bytesRead = from.read(buffer)) != -1) {
                to.write(buffer, 0, bytesRead);
            }
        } finally {
            if (from != null) {
                try {
                    from.close();
                } catch (IOException e) { }
            }
            if (to != null) {
                try {
                    to.close();
                } catch (IOException e) { }
            }
        }
    }

    public static void copyNIO(String fromFileName, String toFileName) throws IOException {
        // Getting file channels
        FileChannel in = null;
        FileChannel out = null;
        try {
            in = new FileInputStream(fromFileName).getChannel();
            out = new FileOutputStream(toFileName).getChannel();
            // JavaVM does its best to do this as native I/O operations.
            in.transferTo(0, in.size(), out);
        }
        finally
        {
            // Closing file channels will close corresponding stream objects as well.
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) { }
            }
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) { }
            }
        }
    }

    public static void copyNIO2(String FileName, String toFileName) throws IOException {
        FileSystem local = FileSystems.getDefault();
        Path from = local.getPath(FileName);
        Path to = local.getPath(toFileName);
        from.copyTo(to);
    }
}
